/**************************************************************

   Program Title:  GenY32 GRBL Port
   
   Program Name:  fs.c	 Version: 1.00
   
   Author:  ALD	  					  Date:  May 10, 2014
   
   Compiled with AVR Studio 6.1.2730
   
   This file contains routines that implement the initialization 
   of the SD file system and basic file read/write operations.
   
   Atmel ASF drivers and FATFS are used for lower layers.
   
   Copyright 2014, Tony da Costa  All rights reserved.
   
****************************************************************/
/*
 * Copyright 2014, Tony da Costa  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TONY DA COSTA AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SOC ROBOTICS, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

/*
 * fs.c
 *
 * Created: 04/05/2014 4:56:44 PM
 *  Author: tdacosta
 */ 
#include "asf.h"
#include "fs.h"

BOOL fs_initialized=FALSE;

BOOL fs_Init(void)
{
	U32 sectors;
	spi_options_t spiopts;
	uint32_t startTime;
	BOOL passed=TRUE;
	
	//Initial FATFS and subsystems
	spiopts.reg=0;
	spiopts.baudrate=12000000;
	spiopts.bits=8;
	spiopts.spck_delay=1;
	spiopts.trans_delay=1;
	spiopts.stay_act=0;
	spiopts.spi_mode=3;
	spiopts.modfdis=1;
	spi_initMaster(SD_MMC_SPI,&spiopts);
	spi_setupChipReg(SD_MMC_SPI,&spiopts,sysclk_get_pba_hz());
	spi_enable(SD_MMC_SPI);
	sd_mmc_spi_init(spiopts, sysclk_get_pba_hz());

	// Wait for SD card to come online
	startTime=timer_GetTickCount();
	while (!sd_mmc_spi_mem_check())
	{
		if ((timer_GetTickCount()-startTime)>1000)
		{
			break;
		}
	};
	if (sd_mmc_spi_mem_check())
	{
		//printf("SD card size=%llu\r\n",capacity);
	}
	else
	{
		//printf("Failed to detect SD card.\r\n");
		passed=FALSE;
	}
	
	// Check the next layer
	startTime=timer_GetTickCount();
	while (mem_test_unit_ready(LUN_ID_SD_MMC_SPI_MEM) != CTRL_GOOD)
	{
		if ((timer_GetTickCount()-startTime)>2000) break;
	}
	if (mem_test_unit_ready(LUN_ID_SD_MMC_SPI_MEM) == CTRL_GOOD)
	{
		// Get and display the capacity
		mem_read_capacity(LUN_ID_SD_MMC_SPI_MEM, &sectors);
		//printf("OK: \t%lu MB\r\n",(sectors + 1) >> (20 - FS_SHIFT_B_TO_SECTOR));
	}
	else
	{
		//printf("Failed to detect SD card.\r\n");
		passed=FALSE;
	}
	
	// reset all navigators
	nav_reset();
	if (!nav_drive_set(0))
	{
		//printf("Failed to set drive.\r\n");
		passed=FALSE;
	}
	if (!nav_partition_mount())
	{
		//printf("Failed to mount partition.\r\n");
		passed=FALSE;
	}
	fs_initialized=passed;
	return passed;
}

BOOL fs_IsInitialized(void)
{
	return fs_initialized;
}

#if 0
static const char *testString="File output test";
BOOL fs_Test(void)
{
	int fileid;
	ssize_t nWritten;

	if (!fs_initialized) return FALSE;
		
	fileid=open("./test.txt", O_CREAT|O_WRONLY);
	if (fileid>=0)
	{
		printf("File opened.\r\n");
		nWritten=write(fileid, testString, strlen(testString)+1);
		if (nWritten==(strlen(testString)+1))
		{
			printf("File write good.\r\n");
		}
		else
		{
			printf("Wrote %i bytes, should have been %lu\r\n",nWritten,strlen(testString)+1);
			
		}
		close(fileid);
	}
	else
	{
		printf("Failed to open file(%i).",fileid);
	}
}
#endif

BOOL fs_WriteWithChecksum(uint16_t findex, char *source, unsigned int size)
{
	int fileid;
	ssize_t nWritten;
	char filename[15];
	unsigned int size2=size;

	if (!fs_initialized) return FALSE;
	if (findex>9999) return FALSE;

	//Compute checksum
	unsigned char checksum = 0;
	char *srcPtr=source;
	for(; size2 > 0; size2--) {
		checksum = (checksum << 1) || (checksum >> 7);
		checksum += *srcPtr++;
	}
	
	//Open and write data
	sprintf(filename,"./grbl%04u.cfg",findex);
	fileid=open(filename, O_CREAT|O_WRONLY);
	if (fileid>=0)
	{
		//printf("File opened.\r\n");
		nWritten=write(fileid, source, size);
		if (nWritten!=size) {close(fileid); return FALSE;};
		nWritten=write(fileid, &checksum, 1);
		if (nWritten!=1) {close(fileid); return FALSE;};
		close(fileid);
	}
	else
	{
		return FALSE;
	}
	return TRUE;	
}

BOOL fs_ReadWithChecksum(uint16_t findex, char *destination, unsigned int size)
{
	int fileid;
	ssize_t nRead;
	char filename[15];
	unsigned char checksumIn;
	unsigned char checksum = 0;
	unsigned int size2=size;

	if (!fs_initialized) return FALSE;
	if (findex>9999) return FALSE;

	//Open and write data
	sprintf(filename,"./grbl%04u.cfg",findex);
	fileid=open(filename, O_RDONLY);
	if (fileid>=0)
	{
		//printf("File opened.\r\n");
		nRead=read(fileid, destination, size);
		if (nRead!=size) {close(fileid); return FALSE;};
		nRead=read(fileid, &checksumIn, 1);
		if (nRead!=1) {close(fileid); return FALSE;};
		close(fileid);

		//Compute checksum
		char *dataPtr=destination;
		for(; size2 > 0; size2--) {
			checksum = (checksum << 1) || (checksum >> 7);
			checksum += *dataPtr++;
		}
	}
	else
	{
		return FALSE;
	}
	return (checksumIn==checksum);	
}