/**************************************************************

   Program Title:  GenY32 GRBL Port
   
   Program Name:  nvram.c	 Version: 1.00
   
   Author:  ALD	  					  Date:  May 11, 2014
   
   Compiled with AVR Studio 6.1.2730
   
   This file contains routines that implement non-volatile storage
   using the last 4KB of on-chip flash.
   
   Atmel ASF drivers are used for lower layers.
   
   Copyright 2014, Tony da Costa  All rights reserved.
   
****************************************************************/
/*
 * Copyright 2014, Tony da Costa  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY TONY DA COSTA AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL SOC ROBOTICS, 
 * INC. OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

/*
 * nvram.c
 *
 * Created: 10/05/2014 8:46:43 PM
 *  Author: tdacosta
 */ 

#include "asf.h"
#include <string.h>

__attribute__((__section__(".flash_nvram")))
static uint8_t nvram_data[NVRAM_SIZE];

BOOL nvram_Init(void)
{
	return TRUE;
}

BOOL nvram_Read(uint16_t address, uint8_t *destPtr, uint16_t size)
{
	if ((address+size)>NVRAM_SIZE) return FALSE;
	if (!destPtr) return FALSE;
	memcpy(destPtr,nvram_data+address,size);
	return TRUE;
}

BOOL nvram_Write(uint16_t address, uint8_t *srcPtr, uint16_t size)
{
	bool irqenabled;

	if ((address+size)>NVRAM_SIZE) return FALSE;
	if (!srcPtr) return FALSE;
	irqenabled=cpu_irq_is_enabled();
	cpu_irq_disable();
	flashc_memcpy(nvram_data+address, srcPtr, size, true);
	if (irqenabled) cpu_irq_enable();
	return flashc_is_programming_error()?FALSE:TRUE;
}

BOOL nvram_ReadWithChecksum(uint16_t address, uint8_t *destPtr, uint16_t size)
{
	if ((address+size+1)>NVRAM_SIZE) return FALSE;
	if (!destPtr) return FALSE;

	unsigned char data, checksum = 0;
	uint16_t source=address;
	for(; size > 0; size--) 
	{
		data = nvram_data[source++];
		checksum = (checksum << 1) || (checksum >> 7);
		checksum += data;
		*(destPtr++) = data;
	}
	return(checksum == nvram_data[source]);
}

BOOL nvram_WriteWithChecksum(uint16_t address, uint8_t *srcPtr, uint16_t size)
{
	bool irqenabled;

	if ((address+size+1)>NVRAM_SIZE) return FALSE;
	if (!srcPtr) return FALSE;

	unsigned char checksum = 0;
	uint16_t size2=size;
	uint8_t *source=srcPtr;
	for(; size2 > 0; size2--) 
	{
		checksum = (checksum << 1) || (checksum >> 7);
		checksum += *source++;
	}
	
	irqenabled=cpu_irq_is_enabled();
	cpu_irq_disable();
	flashc_memcpy(nvram_data+address, srcPtr, size, true);
	if (irqenabled) cpu_irq_enable();
	if (flashc_is_programming_error()) return FALSE;
	
	timer_WaitMs(2); //Let some interrupts run
	
	//This is really a waste--we're actually writing the same page twice just to add the checksum,
	//but without an intermediate buffer to work with, there's not much we can do about that.
	irqenabled=cpu_irq_is_enabled();
	cpu_irq_disable();
	flashc_memcpy(nvram_data+address+size, &checksum, 1, true);
	if (irqenabled) cpu_irq_enable();
	if (flashc_is_programming_error()) return FALSE;

	return TRUE;	
}